#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "Nineword.h"
#include "Board.h"

extern Nineword *nw;
extern bool uppercase;


static board_t board[LETTERS];
static char userword[10];
static int pos;

static void letter_click( GtkWidget *widget, gpointer button )
{

    if( board[(int)button].pressed == 0 ) {
	board[(int)button].pressed = 1;
	userword[pos++] = board[(int)button].letter;
	nw->set_entry_text(userword);
    }

    /* You can only toggle it on, not off again */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(board[(int)button].button),
				 TRUE);

}



/* Make a board and attatch it to table */
Board::Board(GtkWidget *table)
{
    GtkWidget *btn;
    GtkWidget *label;
    GtkStyle *style, *bs, *ws;
    GdkColormap *cmap;
    GdkColor gray;
    GdkFont *fixed_font;
    int i, j;
    int x, y;

    if (!gdk_color_parse("dim gray", &gray)) {
	g_error("couldn't allocate color");
    }

    style = gtk_style_copy( gtk_widget_get_style( table ) );

#if USE_FONTS
    fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
    style->font = fixed_font;
#endif

    /* Black Square */
    bs = gtk_style_copy( style );
    bs->fg[GTK_STATE_NORMAL] = bs->white;
    bs->bg[GTK_STATE_NORMAL] = bs->black;
    bs->fg[GTK_STATE_ACTIVE] = bs->white;
    bs->bg[GTK_STATE_ACTIVE] = gray;

    /* White Square */
    ws = gtk_style_copy( style );
    ws->fg[GTK_STATE_NORMAL] = ws->black;
    ws->bg[GTK_STATE_NORMAL] = ws->white;

    for( i=0 ; i<LETTERS ; i++ ) {
	y = i/3;
	x = i%3;

	/* Middle button */
	if( i == 4 )
	    style = bs;
	else
	    style = ws;

	btn = gtk_toggle_button_new();
	gtk_widget_set_usize( btn, 48, 48);
	gtk_table_attach_defaults( GTK_TABLE(table), btn, x, x+1, y, y+1 );
	board[i].handler_id = gtk_signal_connect(GTK_OBJECT(btn), "clicked",
						 GTK_SIGNAL_FUNC(letter_click),
						 (gpointer) i);
	label = gtk_label_new("");
	gtk_widget_set_style( GTK_WIDGET(label), style );
	gtk_widget_show(label);
	gtk_container_add (GTK_CONTAINER (btn), label);

	gtk_widget_set_style( GTK_WIDGET(btn), style );
	gtk_widget_show(btn);

	board[i].button = btn;
	board[i].label = label;
     }

}


Board::~Board(void)
{
    int i;

    /* Reset entry text */
    nw->set_entry_text("");

    for( i=0 ; i<LETTERS ; i++ )
	gtk_object_destroy( GTK_OBJECT(board[i].button) );

}


void Board::set_word(char *word)
{
    char letter[2] = { 0, 0 };
    int i;

    pos = 0;    /* Pos = position in array userword */
    memset( userword, 0, sizeof(userword) );

    for( i=0 ; i<LETTERS ; i++ ) {
	letter[0] = uppercase? toupper( word[i] ) : word[i];
	gtk_label_set_text( GTK_LABEL(board[i].label), letter );
	gtk_signal_handler_block( GTK_OBJECT(board[i].button),
				  board[i].handler_id );
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(board[i].button),FALSE);
	gtk_signal_handler_unblock( GTK_OBJECT(board[i].button),
				    board[i].handler_id );
	board[i].letter = word[i];
	board[i].pressed = 0;
    }

}


char *Board::get_userword(void)
{
    return userword;
}
