#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Hash.h"
#include "Dictionary.h"
#include "Board.h"
#include "Wordlist.h"
#include "Nineword.h"
#include "Game.h"


#define DICTIONARY "/usr/dict/words"

Nineword *nw;
Dictionary *dict;
Board *board;
Wordlist *wordlist;
GtkWidget *table;
Game *game;
bool uppercase;


int main(int argc, char *argv[])
{

    gtk_init(&argc, &argv);
    dict = new Dictionary(DICTIONARY);
    game = new Game( dict->random_word(LETTERS) );
    nw = new Nineword(); /* Window & Board */
    gtk_main(); /* Event loop */

    /* Clean up */
    delete dict;
    delete game;
    delete nw;

    return 0;

}

static char shuffledword[10];
static void check(GtkWidget *w, gpointer data);
static void clear(GtkWidget *w, gpointer data);
static void new_word(GtkWidget *w, gpointer data);
static void entry_check(GtkWidget *widget, GtkWidget *caller);

struct buttonlabel{
  char *label;
  void (*func)(GtkWidget *, gpointer);
} side_button[3] = {
    { "Check", check },
    { "Clear", clear },
    { "New Word", new_word }
};


/*
  Box cramming order:

  Window
         Hbox {  Table { buttons }  Vbox { buttons }  }         
         Entry
         Wordlist.

*/
Nineword::Nineword(void)
{
     GtkWidget *win;
     GtkWidget *bigbox, *hbox, *vbox, *bottombox;
     GtkWidget *btn;
     GtkWidget *menubar;

     win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(win), "Nineword");
     gtk_signal_connect(GTK_OBJECT(win), "destroy",
			GTK_SIGNAL_FUNC(gtk_exit), NULL);
     gtk_widget_realize(win);
     gtk_container_set_border_width(GTK_CONTAINER(win), 0);

     bigbox = gtk_vbox_new(FALSE, 0);
     gtk_container_add( GTK_CONTAINER(win), GTK_WIDGET(bigbox) );

     menubar = create_menubar(win);
     gtk_box_pack_start( GTK_BOX(bigbox), menubar, FALSE, FALSE, 0);
     gtk_widget_show(menubar);

     // Horizontal box that stores table & Botton (in the vbox)
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start( GTK_BOX(bigbox), hbox, TRUE, TRUE, 10);
     gtk_widget_show(hbox);

     // The text part at the bottom of the buttons
     entry = gtk_entry_new_with_max_length(9);
     gtk_signal_connect(GTK_OBJECT(entry), "activate",
			GTK_SIGNAL_FUNC(entry_check), entry);    

     gtk_box_pack_start( GTK_BOX(bigbox), entry, TRUE, TRUE, 0);
     gtk_widget_show(entry);

     wordlist = new Wordlist(bigbox);

     bottombox = gtk_hbox_new( FALSE, 0 );
     gtk_box_pack_start( GTK_BOX(bigbox), bottombox, TRUE, TRUE, 0);

     // Score
     score_label = gtk_label_new("");
     gtk_box_pack_start( GTK_BOX(bottombox), score_label, TRUE, TRUE, 0);
     gtk_widget_show(score_label);

     // Letters
     letter_label = gtk_label_new("");
     gtk_box_pack_start( GTK_BOX(bottombox), letter_label, TRUE, TRUE, 0);
     gtk_widget_show(letter_label);

     gtk_widget_show( bottombox );
     set_score(0, 0);

     //************  Contents of HBox ************
     table = gtk_table_new(3, 3, TRUE);
     gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 10);
     gtk_widget_show(table);
     
     strncpy( shuffledword, game->shuffle(), 10 );
     board = new Board(table);
     board->set_word( shuffledword );
     
     // VBox holds the buttons
     vbox = gtk_vbox_new(TRUE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 10);
     gtk_widget_show(vbox);

     for( int i=0 ; i<3 ; i++ ) {
	 btn = gtk_button_new_with_label( side_button[i].label );
	 gtk_widget_set_usize(btn, 80, 48);
	 gtk_box_pack_start(GTK_BOX(vbox), btn, TRUE, TRUE, 0);
	 gtk_signal_connect_object(GTK_OBJECT(btn), "clicked",
				   GTK_SIGNAL_FUNC(side_button[i].func),
				   NULL);
	 gtk_widget_show(btn);
     }

     gtk_widget_show(bigbox);
     gtk_widget_show(win);

}

static void menu_game_new(...);
static void menu_uppercase(...);
static void instructions(...);
static void about(...);

static GtkItemFactoryEntry menu_items[] = {
    /* FILE */
    { "/_Game",                 NULL,           NULL, 0,     "<Branch>" },
    { "/Game/_New Game",        "F2",  menu_game_new, 0,           NULL },
    { "/Game/sep1",             NULL,           NULL, 0,  "<Separator>" },
    { "/Game/Quit",     "<control>Q",  gtk_main_quit, 0,          NULL  },
    /* Options */
    { "/_Options",              NULL,           NULL, 0,     "<Branch>" },
    { "/Options/UPPERCASE",     NULL, menu_uppercase, 0, "<ToggleItem>" },
    /* Help */
    { "/_Help",                 NULL,           NULL, 0, "<LastBranch>" },
    { "/_Help/Instructions",    NULL,   instructions, 0,           NULL },
    { "/_Help/About",           NULL,          about, 0,           NULL },
};


GtkWidget *Nineword::create_menubar(GtkWidget  *window)
{
  GtkWidget *menubar;
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

  accel_group = gtk_accel_group_new();

  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
              or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
              the accelerator table while generating menus.
  */

  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
				      accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);

  menubar = gtk_item_factory_get_widget(item_factory, "<main>");

  return menubar;

}


void Nineword::set_score(int score, int letters)
{
    char buf[32];
    
    snprintf(buf, 32, "Score: %d", score);
    gtk_label_set( GTK_LABEL( score_label ), buf );

    snprintf(buf, 32, "Letters: %d", letters);
    gtk_label_set( GTK_LABEL( letter_label ), buf );

}


void Nineword::set_entry_text(char *text)
{

    gtk_entry_set_text( GTK_ENTRY(entry), text);

    /*
      gtk_entry_select_region( GTK_ENTRY(entry), 0,
      GTK_ENTRY(entry)->text_length);  
    */

}


enum {
    ABOUT,
    INSTRUCTIONS
};

static char *textstuff[2][2] = {
    { { "About Nineword:" },
      { "By David Lawrence - philaw@camtech.net.au\n\n"
	"This game is based on a word puzzle called \"Target\"\n"
	"which appears in the Adelaide newspaper \"The Advertiser.\"\n\n"
        VERSION } },
    { { "Instructions:" },
      { "Make as many words of 4 letters or more as possible\n"
	"conforming to the following rules:\n\n"
	"- Each letter must be used once only\n"
	"- Each word must contain the center letter\n"
	"- The word must not end in 'S' or have inital capitals\n"
	"- There is at least one nine letter word\n"
	"- The word can be found in the file \"" DICTIONARY "\"" } }
};


void Nineword::helpmenu(int i)
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *frame;
  GtkWidget *label;

  GtkWidget *buttons_box;
  GtkWidget *ok_button;

  window = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_window_set_title (GTK_WINDOW (window), "Nineword");

  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC (gtk_widget_destroyed), &window);
  gtk_container_border_width (GTK_CONTAINER (window), 10);

  vbox = gtk_vbox_new (FALSE, 10);
  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_widget_show (vbox);

  // Frame
  frame = gtk_frame_new(NULL);
  //gtk_container_border_width( GTK_CONTAINER(frame), 10);
  gtk_frame_set_label( GTK_FRAME(frame), textstuff[i][0] );  
  gtk_frame_set_label_align( GTK_FRAME(frame), 0.1, 0.0);  
  gtk_frame_set_shadow_type( GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);  
  gtk_box_pack_start( GTK_BOX(vbox), frame, FALSE, FALSE, 0);
  gtk_widget_show(frame); 

  // About Nineword....
  label = gtk_label_new( textstuff[i][1] );
  gtk_label_set_justify( GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  //gtk_label_set_line_wrap( GTK_LABEL(label), TRUE);
  gtk_container_add( GTK_CONTAINER(frame), label );
  gtk_widget_show(label);
  
  buttons_box = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start( GTK_BOX(vbox), buttons_box, FALSE, FALSE, 0);
  gtk_widget_show (buttons_box);

  ok_button = gtk_button_new_with_label (" OK ");
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (window));
  gtk_box_pack_start (GTK_BOX (buttons_box), ok_button, TRUE, FALSE, 0);
  gtk_widget_show (ok_button);

  gtk_widget_show (window);

}



static void check(GtkWidget *w, gpointer data)
{
    if( game->check( board->get_userword() ) ) {
	wordlist->add( board->get_userword() );
	nw->set_score( game->score, game->letters );
    }
    clear(NULL, NULL);

}


static void clear(GtkWidget *w, gpointer data)
{

    board->set_word(shuffledword);
    nw->set_entry_text("");

}


static void new_word(GtkWidget *w, gpointer data)
{
    nw->set_entry_text( game->nineword );

    delete game;
    game = new Game( dict->random_word(LETTERS) );
    nw->set_score( game->score, game->letters );
    strcpy( shuffledword, game->shuffle() );
    board->set_word(shuffledword);
    wordlist->clear();

}


void entry_check(GtkWidget *widget, GtkWidget *caller)
{
    char *str;

    str = gtk_entry_get_text( GTK_ENTRY(caller) );
    if( game->check( str ) )
	wordlist->add( str );

    clear(NULL, NULL);

}


static void menu_game_new(...)
{

    /* Pretend that we clicked on the new button. */
    new_word(NULL, NULL);

}

static void menu_uppercase(...)
{

    uppercase = !uppercase;
    /* Pretend that we clicked on the clear button. */
    clear(NULL, NULL);

}





static void about(...)
{
    nw->helpmenu(ABOUT);
}

static void instructions(...)
{
    nw->helpmenu(INSTRUCTIONS);
}





